@@ -19,7 +19,8 @@
 
 #include <spa/param/audio/layout.h>
 #include <spa/param/audio/format-utils.h>
-#include <spa/param/audio/type-info.h>
+#include <spa/param/audio/raw-json.h>
+#include <spa/utils/type-info.h>
 #include <spa/param/tag-utils.h>
 #include <spa/param/props.h>
 #include <spa/utils/result.h>
@@ -27,6 +28,7 @@
 #include <spa/utils/json.h>
 #include <spa/debug/types.h>
 #include <spa/debug/file.h>
+#include <spa/control/ump-utils.h>
 
 #include <pipewire/pipewire.h>
 #include <pipewire/i18n.h>
@@ -78,8 +80,8 @@ struct data;
 typedef int (*fill_fn)(struct data *d, void *dest, unsigned int n_frames, bool *null_frame);
 
 struct channelmap {
-	int n_channels;
-	int channels[SPA_AUDIO_MAX_CHANNELS];
+	uint32_t n_channels;
+	uint32_t channels[SPA_AUDIO_MAX_CHANNELS];
 };
 
 struct data {
@@ -102,6 +104,7 @@ struct data {
 #define TYPE_ENCODED    3
 #endif
 	int data_type;
+	bool raw;
 	const char *remote_name;
 	const char *media_type;
 	const char *media_category;
@@ -117,7 +120,7 @@ struct data {
 
 	unsigned int bitrate;
 	unsigned int rate;
-	int channels;
+	uint32_t channels;
 	struct channelmap channelmap;
 	unsigned int stride;
 	enum unit latency_unit;
@@ -564,10 +567,10 @@ static int channelmap_from_sf(struct channelmap *map)
 		[SF_CHANNEL_MAP_TOP_REAR_RIGHT] =        SPA_AUDIO_CHANNEL_TRR,
 		[SF_CHANNEL_MAP_TOP_REAR_CENTER] =       SPA_AUDIO_CHANNEL_TRC
 	};
-	int i;
+	uint32_t i;
 
 	for (i = 0; i < map->n_channels; i++) {
-		if (map->channels[i] >= 0 && map->channels[i] < (int) SPA_N_ELEMENTS(table))
+		if (map->channels[i] < SPA_N_ELEMENTS(table))
 			map->channels[i] = table[map->channels[i]];
 		else
 			map->channels[i] = SPA_AUDIO_CHANNEL_UNKNOWN;
@@ -576,8 +579,8 @@ static int channelmap_from_sf(struct channelmap *map)
 }
 struct mapping {
 	const char *name;
-	unsigned int channels;
-	unsigned int values[32];
+	uint32_t channels;
+	uint32_t values[32];
 };
 
 static const struct mapping maps[] =
@@ -597,21 +600,8 @@ static const struct mapping maps[] =
 	{ "surround-71",  SPA_AUDIO_LAYOUT_7_1 },
 };
 
-static unsigned int find_channel(const char *name)
-{
-	int i;
-
-	for (i = 0; spa_type_audio_channel[i].name; i++) {
-		if (spa_streq(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)))
-			return spa_type_audio_channel[i].type;
-	}
-	return SPA_AUDIO_CHANNEL_UNKNOWN;
-}
-
 static int parse_channelmap(const char *channel_map, struct channelmap *map)
 {
-	int i, nch;
-
 	SPA_FOR_EACH_ELEMENT_VAR(maps, m) {
 		if (spa_streq(m->name, channel_map)) {
 			map->n_channels = m->channels;
@@ -621,16 +611,7 @@ static int parse_channelmap(const char *channel_map, struct channelmap *map)
 		}
 	}
 
-	spa_auto(pw_strv) ch = pw_split_strv(channel_map, ",", SPA_AUDIO_MAX_CHANNELS, &nch);
-	if (ch == NULL)
-		return -1;
-
-	map->n_channels = nch;
-	for (i = 0; i < map->n_channels; i++) {
-		int c = find_channel(ch[i]);
-		map->channels[i] = c;
-	}
-
+	spa_audio_parse_position(channel_map, strlen(channel_map), map->channels, &map->n_channels);
 	return 0;
 }
 
@@ -671,13 +652,11 @@ static int channelmap_default(struct channelmap *map, int n_channels)
 
 static void channelmap_print(struct channelmap *map)
 {
-	int i;
+	uint32_t i;
 
 	for (i = 0; i < map->n_channels; i++) {
-		const char *name = spa_debug_type_find_name(spa_type_audio_channel, map->channels[i]);
-		if (name == NULL)
-			name = ":UNK";
-		printf("%s%s", spa_debug_type_short_name(name), i + 1 < map->n_channels ? "," : "");
+		const char *name = spa_type_audio_channel_to_short_name(map->channels[i]);
+		fprintf(stderr, "%s%s", name, i + 1 < map->n_channels ? "," : "");
 	}
 }
 
@@ -686,7 +665,7 @@ static void on_core_info(void *userdata, const struct pw_core_info *info)
 	struct data *data = userdata;
 
 	if (data->verbose)
-		printf("remote %"PRIu32" is named \"%s\"\n",
+		fprintf(stderr, "remote %"PRIu32" is named \"%s\"\n",
 				info->id, info->name);
 }
 
@@ -715,7 +694,7 @@ on_state_changed(void *userdata, enum pw_stream_state old,
 	int ret;
 
 	if (data->verbose)
-		printf("stream state changed %s -> %s\n",
+		fprintf(stderr, "stream state changed %s -> %s\n",
 				pw_stream_state_as_string(old),
 				pw_stream_state_as_string(state));
 
@@ -726,7 +705,7 @@ on_state_changed(void *userdata, enum pw_stream_state old,
 					SPA_PROP_volume, 1, &data->volume,
 					0);
 			if (data->verbose)
-				printf("stream set volume to %.3f - %s\n", data->volume,
+				fprintf(stderr, "stream set volume to %.3f - %s\n", data->volume,
 						ret == 0 ? "success" : "FAILED");
 
 			data->volume_is_set = true;
@@ -735,7 +714,7 @@ on_state_changed(void *userdata, enum pw_stream_state old,
 			struct timespec timeout = {0, 1}, interval = {1, 0};
 			struct pw_loop *l = pw_main_loop_get_loop(data->loop);
 			pw_loop_update_timer(l, data->timer, &timeout, &interval, false);
-			printf("stream node %"PRIu32"\n",
+			fprintf(stderr, "stream node %"PRIu32"\n",
 				pw_stream_get_node_id(data->stream));
 		}
 		break;
@@ -747,13 +726,13 @@ on_state_changed(void *userdata, enum pw_stream_state old,
 		}
 		break;
 	case PW_STREAM_STATE_ERROR:
-		printf("stream node %"PRIu32" error: %s\n",
+		fprintf(stderr, "stream node %"PRIu32" error: %s\n",
 				pw_stream_get_node_id(data->stream),
 				error);
 		pw_main_loop_quit(data->loop);
 		break;
 	case PW_STREAM_STATE_UNCONNECTED:
-		printf("stream node %"PRIu32" unconnected\n",
+		fprintf(stderr, "stream node %"PRIu32" unconnected\n",
 				pw_stream_get_node_id(data->stream));
 		pw_main_loop_quit(data->loop);
 		break;
@@ -784,7 +763,7 @@ on_param_changed(void *userdata, uint32_t id, const struct spa_pod *param)
 	int err;
 
 	if (data->verbose)
-		printf("stream param change: %s\n",
+		fprintf(stderr, "stream param change: %s\n",
 			spa_debug_type_find_name(spa_type_param, id));
 
 	if (id != SPA_PARAM_Format || param == NULL)
@@ -811,7 +790,7 @@ on_param_changed(void *userdata, uint32_t id, const struct spa_pod *param)
 	data->stride = data->dsf.layout.channels * SPA_ABS(data->dsf.layout.interleave);
 
 	if (data->verbose) {
-		printf("DSD: channels:%d bitorder:%s interleave:%d stride:%d\n",
+		fprintf(stderr, "DSD: channels:%d bitorder:%s interleave:%d stride:%d\n",
 				data->dsf.layout.channels,
 				data->dsf.layout.lsb ? "lsb" : "msb",
 				data->dsf.layout.interleave,
@@ -873,7 +852,7 @@ static void on_process(void *userdata)
 			fprintf(stderr, "fill error %d\n", n_fill_frames);
 		} else {
 			if (data->verbose)
-				printf("drain start\n");
+				fprintf(stderr, "drain start\n");
 		}
 	} else {
 		bool null_frame = false;
@@ -904,7 +883,7 @@ static void on_drained(void *userdata)
 	struct data *data = userdata;
 
 	if (data->verbose)
-		printf("stream drained\n");
+		fprintf(stderr, "stream drained\n");
 
 	data->drained = true;
 	pw_main_loop_quit(data->loop);
@@ -930,7 +909,7 @@ static void do_print_delay(void *userdata, uint64_t expirations)
 	struct data *data = userdata;
 	struct pw_time time;
 	pw_stream_get_time_n(data->stream, &time, sizeof(time));
-	printf("stream time: now:%"PRIi64" rate:%u/%u ticks:%"PRIu64
+	fprintf(stderr, "stream time: now:%"PRIi64" rate:%u/%u ticks:%"PRIu64
 			" delay:%"PRIi64" queued:%"PRIu64
 			" buffered:%"PRIi64" buffers:%u avail:%u size:%"PRIu64"\n",
 		time.now,
@@ -961,6 +940,8 @@ static const struct option long_options[] = {
 	{ "record",		no_argument,	   NULL, 'r' },
 	{ "playback",		no_argument,	   NULL, 'p' },
 	{ "midi",		no_argument,	   NULL, 'm' },
+	{ "dsd",		no_argument,	   NULL, 'd' },
+	{ "encoded",		no_argument,	   NULL, 'o' },
 
 	{ "remote",		required_argument, NULL, 'R' },
 
@@ -977,6 +958,7 @@ static const struct option long_options[] = {
 	{ "format",		required_argument, NULL, OPT_FORMAT },
 	{ "volume",		required_argument, NULL, OPT_VOLUME },
 	{ "quality",		required_argument, NULL, 'q' },
+	{ "raw",		no_argument, NULL, 'a' },
 
 	{ NULL, 0, NULL, 0 }
 };
@@ -1021,6 +1003,7 @@ static void show_usage(const char *name, bool is_error)
 	     "      --format                          Sample format %s (req. for rec) (default %s)\n"
 	     "      --volume                          Stream volume 0-1.0 (default %.3f)\n"
 	     "  -q  --quality                         Resampler quality (0 - 15) (default %d)\n"
+	     "  -a, --raw                             RAW mode\n"
 	     "\n"),
 	     DEFAULT_RATE,
 	     DEFAULT_CHANNELS,
@@ -1080,13 +1063,36 @@ static int midi_play(struct data *d, void *src, unsigned int n_frames, bool *nul
 		midi_file_read_event(d->midi.file, &ev);
 
 		if (d->verbose)
-			midi_file_dump_event(stdout, &ev);
+			midi_file_dump_event(stderr, &ev);
 
-		if (ev.data[0] == 0xff)
+		if (ev.type == MIDI_EVENT_TYPE_MIDI1) {
+			size_t size;
+			uint8_t *data;
+			uint64_t state = 0;
+
+			if (ev.data[0] == 0xff)
+				continue;
+
+			data = ev.data;
+			size = ev.size;
+
+			while (size > 0) {
+				uint32_t ump[4];
+				int ump_size = spa_ump_from_midi(&data, &size,
+						ump, sizeof(ump), 0, &state);
+				if (ump_size <= 0)
+					break;
+
+				spa_pod_builder_control(&b, frame, SPA_CONTROL_UMP);
+				spa_pod_builder_bytes(&b, ump, ump_size);
+			}
+		} else if (ev.type == MIDI_EVENT_TYPE_UMP) {
+			spa_pod_builder_control(&b, frame, SPA_CONTROL_UMP);
+			spa_pod_builder_bytes(&b, ev.data, ev.size);
+		}
+		else
 			continue;
 
-		spa_pod_builder_control(&b, frame, SPA_CONTROL_Midi);
-		spa_pod_builder_bytes(&b, ev.data, ev.size);
 		have_data = true;
 	}
 	spa_pod_builder_pop(&b, &f);
@@ -1111,16 +1117,17 @@ static int midi_record(struct data *d, void *src, unsigned int n_frames, bool *n
 	SPA_POD_SEQUENCE_FOREACH((struct spa_pod_sequence*)pod, c) {
 		struct midi_event ev;
 
-		if (c->type != SPA_CONTROL_Midi)
+		if (c->type != SPA_CONTROL_UMP)
 			continue;
 
 		ev.track = 0;
 		ev.sec = (frame + c->offset) / (float) d->position->clock.rate.denom;
 		ev.data = SPA_POD_BODY(&c->value),
 		ev.size = SPA_POD_BODY_SIZE(&c->value);
+		ev.type = MIDI_EVENT_TYPE_UMP;
 
 		if (d->verbose)
-			midi_file_dump_event(stdout, &ev);
+			midi_file_dump_event(stderr, &ev);
 
 		midi_file_write_event(d->midi.file, &ev);
 	}
@@ -1145,7 +1152,7 @@ static int setup_midifile(struct data *data)
 	}
 
 	if (data->verbose)
-		printf("midifile: opened file \"%s\" format %08x ntracks:%d div:%d\n",
+		fprintf(stderr, "midifile: opened file \"%s\" format %08x ntracks:%d div:%d\n",
 				data->filename,
 				data->midi.info.format, data->midi.info.ntracks,
 				data->midi.info.division);
@@ -1196,7 +1203,7 @@ static int setup_dsdfile(struct data *data)
 
 	if (data->dsf.file != NULL) {
 		if (data->verbose)
-			printf("dsffile: opened file \"%s\" channels:%d rate:%d "
+			fprintf(stderr, "dsffile: opened file \"%s\" channels:%d rate:%d "
 					"samples:%"PRIu64" bitorder:%s\n",
 				data->filename,
 				data->dsf.info.channels, data->dsf.info.rate,
@@ -1206,7 +1213,7 @@ static int setup_dsdfile(struct data *data)
 		data->fill = dsf_play;
 	} else {
 		if (data->verbose)
-			printf("dfffile: opened file \"%s\" channels:%d rate:%d "
+			fprintf(stderr, "dfffile: opened file \"%s\" channels:%d rate:%d "
 					"samples:%"PRIu64" bitorder:%s\n",
 				data->filename,
 				data->dff.info.channels, data->dff.info.rate,
@@ -1250,7 +1257,7 @@ static int setup_pipe(struct data *data)
 	data->fill = data->mode == mode_playback ?  stdin_play : stdout_record;
 
 	if (data->verbose)
-		printf("PIPE: rate=%u channels=%u fmt=%s samplesize=%u stride=%u\n",
+		fprintf(stderr, "PIPE: rate=%u channels=%u fmt=%s samplesize=%u stride=%u\n",
 				data->rate, data->channels,
 				info->name, info->width, data->stride);
 
@@ -1420,7 +1427,7 @@ static int setup_encodedfile(struct data *data)
 	data->fill = encoded_playback_fill;
 
 	if (data->verbose) {
-		printf("Opened file \"%s\" with encoded audio; channels:%d rate:%d bitrate: %d time units %d/%d\n",
+		fprintf(stderr, "Opened file \"%s\" with encoded audio; channels:%d rate:%d bitrate: %d time units %d/%d\n",
 		       data->filename, data->channels, data->rate, data->bitrate,
 		       data->encoded.audio_stream->time_base.num, data->encoded.audio_stream->time_base.den);
 	}
@@ -1467,9 +1474,9 @@ static int setup_sndfile(struct data *data)
 	}
 
 	if (data->verbose)
-		printf("sndfile: opened file \"%s\" format %08x channels:%d rate:%d\n",
+		fprintf(stderr, "sndfile: opened file \"%s\" format %08x channels:%d rate:%d\n",
 				data->filename, info.format, info.channels, info.samplerate);
-	if (data->channels > 0 && info.channels != data->channels) {
+	if (data->channels > 0 && info.channels != (int)data->channels) {
 		fprintf(stderr, "sndfile: given channels (%u) don't match file channels (%d)\n",
 				data->channels, info.channels);
 		return -EINVAL;
@@ -1494,9 +1501,9 @@ static int setup_sndfile(struct data *data)
 				def = true;
 			}
 			if (data->verbose) {
-				printf("sndfile: using %s channel map: ", def ? "default" : "file");
+				fprintf(stderr, "sndfile: using %s channel map: ", def ? "default" : "file");
 				channelmap_print(&data->channelmap);
-				printf("\n");
+				fprintf(stderr, "\n");
 			}
 		}
 		fill_properties(data);
@@ -1510,7 +1517,7 @@ static int setup_sndfile(struct data *data)
 		return -EIO;
 
 	if (data->verbose)
-		printf("PCM: fmt:%s rate:%u channels:%u width:%u\n",
+		fprintf(stderr, "PCM: fmt:%s rate:%u channels:%u width:%u\n",
 				fi->name, data->rate, data->channels, fi->width);
 
 	/* we read and write S24 as S32 with sndfile */
@@ -1590,7 +1597,7 @@ static int setup_properties(struct data *data)
 	}
 
 	if (data->verbose)
-		printf("rate:%d latency:%u (%.3fs)\n",
+		fprintf(stderr, "rate:%d latency:%u (%.3fs)\n",
 				data->rate, nom, data->rate ? (double)nom/data->rate : 0.0f);
 	if (nom && pw_properties_get(data->props, PW_KEY_NODE_LATENCY) == NULL)
 		pw_properties_setf(data->props, PW_KEY_NODE_LATENCY, "%u/%u", nom, data->rate);
@@ -1664,9 +1671,9 @@ int main(int argc, char *argv[])
 	}
 
 #ifdef HAVE_PW_CAT_FFMPEG_INTEGRATION
-	while ((c = getopt_long(argc, argv, "hvprmdoR:q:P:", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "hvprmdoR:q:P:a", long_options, NULL)) != -1) {
 #else
-	while ((c = getopt_long(argc, argv, "hvprmdR:q:P:", long_options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "hvprmdR:q:P:a", long_options, NULL)) != -1) {
 #endif
 
 		switch (c) {
@@ -1718,6 +1725,10 @@ int main(int argc, char *argv[])
 			data.quality = atoi(optarg);
 			break;
 
+		case 'a':
+			data.raw = true;
+			break;
+
 		case OPT_MEDIA_TYPE:
 			data.media_type = optarg;
 			break;
@@ -1846,11 +1857,7 @@ int main(int argc, char *argv[])
 	pw_loop_add_signal(l, SIGINT, do_quit, &data);
 	pw_loop_add_signal(l, SIGTERM, do_quit, &data);
 
-	data.context = pw_context_new(l,
-			pw_properties_new(
-				PW_KEY_CONFIG_NAME, "client-rt.conf",
-				NULL),
-			0);
+	data.context = pw_context_new(l, NULL, 0);
 	if (!data.context) {
 		fprintf(stderr, "error: pw_context_new() failed: %m\n");
 		goto error_no_context;
@@ -1867,7 +1874,7 @@ int main(int argc, char *argv[])
 	}
 	pw_core_add_listener(data.core, &data.core_listener, &core_events, &data);
 
-	if (spa_streq(data.filename, "-")) {
+	if (data.raw) {
 		ret = setup_pipe(&data);
 	} else {
 		switch (data.data_type) {
@@ -1952,7 +1959,7 @@ int main(int argc, char *argv[])
 				SPA_FORMAT_mediaType,		SPA_POD_Id(SPA_MEDIA_TYPE_application),
 				SPA_FORMAT_mediaSubtype,	SPA_POD_Id(SPA_MEDIA_SUBTYPE_control));
 
-		pw_properties_set(data.props, PW_KEY_FORMAT_DSP, "8 bit raw midi");
+		pw_properties_set(data.props, PW_KEY_FORMAT_DSP, "32 bit raw UMP");
 		break;
 	case TYPE_DSD:
 	{
@@ -2008,7 +2015,7 @@ int main(int argc, char *argv[])
 	pw_stream_add_listener(data.stream, &data.stream_listener, &stream_events, &data);
 
 	if (data.verbose)
-		printf("connecting %s stream; target=%s\n",
+		fprintf(stderr, "connecting %s stream; target=%s\n",
 				data.mode == mode_playback ? "playback" : "record",
 				data.target);
 
@@ -2032,11 +2039,11 @@ int main(int argc, char *argv[])
 		const char *key, *val;
 
 		if ((props = pw_stream_get_properties(data.stream)) != NULL) {
-			printf("stream properties:\n");
+			fprintf(stderr, "stream properties:\n");
 			pstate = NULL;
 			while ((key = pw_properties_iterate(props, &pstate)) != NULL &&
 				(val = pw_properties_get(props, key)) != NULL) {
-				printf("\t%s = \"%s\"\n", key, val);
+				fprintf(stderr, "\t%s = \"%s\"\n", key, val);
 			}
 		}
 	}
